#!/usr/bin/python3

import os
import sys
import shutil
import fnmatch
import re

def pause():
    #Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")

def print_format(text,value):
    #Substitute {0} in text with value.
    print(text.format(value))

def get_files(path,value):
    matches = [] 
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files,value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches

def get_folders(path,value):
    matches = []
    for root, dirs, files in os.walk(path):
        for dirs in fnmatch.filter(files,value):
            matches.append(dirs)
    # Sort folder alphabetically
    matches.sort()

    return matches

def ensure_dir(path):     
    if not os.path.exists(path):
        os.makedirs(path)

def list_files(path,value):
    matches = get_files(path,value)
    print_list(matches)

def print_list(matches):
    divisor = 6
    result = ''
 
    for i in range(int(len(matches)/divisor)):
            j = divisor * (i *1)
            result = ''
            for j in range(j,j+(divisor)):
                result=result+matches[j]+'\t'
            print(result)
    
    k = len(matches) % divisor
    l = (int(len(matches) / divisor)*divisor)

    result = ''
    for m in range(l,len(matches)):
        result=result+matches[m]+'\t'
    print(result)

def mov_files(path,dpath,value):    
    # Create destination folder if it doesn't exist.
    ensure_dir(dpath)
    
    #list_files(path,value)    
    #confirm=input("Enter Y to proceed: ")
    confirm = 'y'
    
    if confirm.lower() == 'y':
        try:
            matches = get_files(path,value)
            for i in range(len(matches)):
                source = os.path.join(path,matches[i])
                if os.path.isfile(source):
                    print("Moving: ", matches[i])
                    dest = os.path.join(dpath,matches[i])
                    shutil.move(source,dest)
            print("Move complete.")
        except:
            print("Problem moving files.")

    #pause()

def get_prefixes(files):
    matches = []
    t = []
    r = re.compile("([a-zA-Z]+[0-9]+-)|([a-zA-Z]+)")

    for file in files:
        m = r.match(file)
        if m:
            if (m.group(1)):
                t.append(m.group(1))
            else:
                t.append(m.group(2))

    matches = list(set(t))
    matches.sort(reverse=True)
    
    #print_list(matches)
    #print()
    #pause()
    
    return matches

def main(argv):
    if len(argv) < 2:
        sys.stderr.write("Usage: %s <folder_path>" % (argv[0],))
        return 1

    if not os.path.exists(argv[1]):
        sys.stderr.write("ERROR: Folder %r was not found!" % (argv[1],))
        return 1
    
    source = argv[1]
    
    files = get_files(source,"*.txt")
    prefs = get_prefixes(files)
    
    for p in prefs:
        d = p.split("-")[0]
        dest = os.path.join(source,d)
        print_format("Moving files into folder: {0}",d)
        mov_files(source,dest,p+"*.txt")

if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
import os
import sys
import shutil
import fnmatch
import subprocess
import argparse
from datetime import date
from collections import defaultdict
from configparser import SafeConfigParser
from fanfic_scraper import current_fanfic
import tldextract
from urllib.parse import urlparse
import requests
from pony.orm import *
from fanfic_scraper.db_pony import DataBaseLogic, Category, Fanfic
import sqlite3

basePath = '/home/ubuntu/OneDrive/'
arcRoot = 'FF_Archive'
db = None
db_name = 'FanfictionDB.db'
db_folder = 'Read'
dl_folder = 'htm'

def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)

def get_config():
    #create config dir if it doesn't exist and return path to config.
    home = os.path.expanduser("~")
    cfg = os.path.join(home,".ff")
    ensure_dir(cfg)

    return os.path.join(cfg,"config.ini")

def config_load():
    global basePath
    global arcRoot
    global db_folder
    global db_name
    global dl_folder
    t_path = ''
    t_root = ''
    t_dbname = ''
    t_dbfolder = ''
    t_dlfolder = ''
    config = SafeConfigParser()

    cfg = get_config()

    if os.path.isfile(cfg):
        config.read(cfg)

        try:
            t_path = config.get('path','path')
            t_root = config.get('path','rootfolder')
            t_dbfolder = config.get('path','dbfolder')
            t_dbname = config.get('path','dbname')
            t_dlfolder = config.get('path','downloadfolder')
        except:
            i=0

        if t_path:
            basePath = t_path
        if t_root:
            arcRoot = t_root
        if t_dbfolder:
            db_folder = t_dbfolder
        if t_dbname:
            db_name = t_dbname
        if t_dlfolder:
            dl_folder = t_dlfolder

def config_save():
    cfg = get_config()

    config = SafeConfigParser()

    config.read(cfg)

    if not config.has_section('path'):
        config.add_section('path')

    config.set('path','path',basePath)
    config.set('path','rootfolder',arcRoot)
    config.set('path','dbfolder',db_folder)
    config.set('path','dbname',db_name)
    config.set('path','downloadfolder',dl_folder)

    with open(cfg, 'w') as f:
        config.write(f)

def IsNotStale(update_date, last_checked):
    wait = 0
    ret = False

    if not last_checked:
        ret = True

    if not ret:
        if not update_date:
            ret = True

    if not ret:

        d0 = update_date
        d1 = last_checked
        delta = d1 - d0

        if delta.days > 360:
            wait = 180
        if delta.days > 180:
            wait = 90
        if delta.days > 90:
            wait = 60
        if delta.days > 60:
            wait = 30
        if delta.days > 30:
            wait = 7
        if delta.days < 30:
            wait = 0

        d0 = last_checked
        d1 = date.today()
        delta = d1 - d0

        if delta.days > wait and wait > 0:
            ret = False
        else:
            ret = True

    return ret

def check_url(url):
    ret = False

    if 'fanfiction.net' in url:
        return True

    return ret

def validate_https(url):

    urlscheme = urlparse(url)
    verify_https = False
    if urlscheme.scheme == 'https':
        try:
            requests.get(url)
            verify_https = True
        except requests.exceptions.SSLError:
            verify_https = False
            print('Could not validate https certificate for url:' +
                  '%s. Proceeding with Insecure certificate.' % (url))
            requests.packages.urllib3.disable_warnings(
                category=InsecureRequestWarning)

    return verify_https

def set_ffargs(location,folder):

    parser = argparse.ArgumentParser(
    description=('fanfic_scraper args'))

    parser.add_argument(
        "-f", "--folder", required=True, help="Name of folder to place fanfic in.")
    parser.add_argument(
        "-l", "--location", default=os.getcwd(), help="set download location")
    #parser.add_argument(
    #    "-c", "--chapters", default=False,
    #    help="Specify chapters to download separated by : (10:15).")
    parser.add_argument(
        "-ct", "--chapterthreads", default=5,
        help="Number of parallel chapters downloads.")
    parser.add_argument(
        "-wt", "--waittime", default=15,
        help="Wait time before retry if encountered with an error")
    parser.add_argument(
        "-rt", "--retries", default=10,
        help="Number of retries before giving up")

    args = parser.parse_args(['--location='+location,'--folder='+folder])

    return args

def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches)/divisor)):
            j = divisor * (i *1)
            result = ''
            for j in range(j,j+(divisor)):
                result=result+matches[j]+'\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor)*divisor)

    result = ''
    for m in range(l,len(matches)):
        result=result+matches[m]+'\t'
    print(result)

def get_subfolders(path):

    subfolders = [f.name for f in os.scandir(path) if f.is_dir() ]
    subfolders.sort()

    return subfolders

def get_files(path,value):
    matches = []
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files,value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches

def convert_file(source,htm):

    txt = htm.split('.')[0]+'.txt'
    ifile = [os.path.abspath(os.path.join(source,htm))]
    ofile = os.path.abspath(os.path.join(source,txt))
    cmd = ['html2text','--ignore-emphasis','-b 80']
    msg = 'Converting file: {0} to text file: {1}'

    print(msg.format(htm,txt))

    with open(ofile, "w") as outfile:
        subprocess.call(cmd+ifile , stdout=outfile)

    for i in ifile:
        os.remove(i)

class FanficDB:

    def __init__(self):
        db_path = os.path.join(basePath,arcRoot,db_folder,db_name)
        dbhandle = DataBaseLogic(db_path)

    def get_categories(self):
        d = defaultdict(list)

        with db_session:
            categories = select(c for c in Category).order_by(Category.Name)

            for c in categories:
                for k,v in c.to_dict().items():
                    d[k].append(v)

        return d['Name']

    def get_cat_id(self, categoryName):

        with db_session:
            cat = Category.get(Name=categoryName)
            ret = cat.Id

        return ret

    def get_fanfics(self,cat_id):
        d = defaultdict(list)
        ret = []

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Abandoned == 0
                        and f.Complete == 0
                        and f.Category_Id == cat_id)
            ff = ff.order_by(Fanfic.Folder)

            for f in ff:
                d = defaultdict(list)
                for k,v in f.to_dict().items():
                    d[k].append(v)
                ret.append(d)

        return ret

    def update_chapter_count(self, fic_id, chapters):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Count = chapters

    def update_last_checked(self, fic_id, last):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Last_Checked = last

    def update_last_updated(self, fic_id, update):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Update_Date = update

def download_stories(args):

    cat = db.get_categories()

    for cat_name in cat:
        #print("Processing:",cat_name)
        download_by_category(cat_name)
        convert_by_category(cat_name)

def download_by_category(category):

    potential_keys = []

    cat_folder = category.replace(' ','_')

    location = os.path.join(basePath,arcRoot,dl_folder,cat_folder)

    cat_id = db.get_cat_id(category)

    fics = db.get_fanfics(cat_id)

    fic_idx = 0
    fic_count = len(fics)
    msg = "Processing: {0} - {1} of {2}."

    for fic in fics:
        fic_idx = fic_idx + 1
        print(msg.format(category,fic_idx,fic_count),end='\r')
        title = fic['Title'][0]
        author = fic['Author'][0]
        folder = fic['Folder'][0]
        chapters = fic['Count'][0]
        update = fic['Update_Date'][0]
        last = fic['Last_Checked'][0]
        if fic['Internet'][0]:
            tmp = fic['Internet'][0]
            tmp2 = tmp.split('#')
            if len(tmp2) > 1:
                url = fic['Internet'][0].split('#')[1]
            else:
                url = ''
        else:
            url = ''
        storyid = fic['StoryId'][0]
        fic_id = fic['Id'][0]

        date_check = IsNotStale(update,last)
        url_check = check_url(url)

        if url_check == True and date_check == True:

            ffargs = set_ffargs(location,folder)

            next_chapter = chapters + 1

            #Initialize class so we can retrieve actual story url
            fanfic = current_fanfic.fanfic(url, ffargs, False)
            #Get fanfic url
            url = fanfic.get_story_url(storyid)
            #Check to see if https verification needs to be disabled.
            verify_https = validate_https(url)
            #Initialize class with correct url.
            fanfic = current_fanfic.fanfic(url, ffargs, verify_https)
            #Download list of chapters
            fanfic.get_chapters()
            # Get current chapter count from site
            cc = fanfic.chapter_count

            if cc >= next_chapter:
                if cc == next_chapter:
                    potential_keys = [next_chapter]
                else:
                    potential_keys = [
                        i * 1 for i in range(next_chapter,(cc + 1))
                        ]
                #Show story about to be updated.
                update = fanfic.get_update_date()
                print('',end='\n')
                print(title, '/', folder, '/', update)
                #Set chapters to download
                fanfic.set_download_chapters(potential_keys)
                #Download Story
                fanfic.download_fanfic()
                print('Downloaded fanfic: ',category,'/',title,'/',folder)
                db.update_chapter_count(fic_id,cc)
                db.update_last_updated(fic_id,update)
                db.update_last_checked(fic_id,date.today())
            else:
                db.update_last_checked(fic_id,date.today())

    print('',end='\n')

def convert_by_category(category):

    cat_folder = category.replace(' ','_')
    location = os.path.join(basePath,arcRoot,dl_folder,cat_folder)

    folders = get_subfolders(location)

    for folder in folders:
        target = os.path.realpath(os.path.join(location,folder))
        files = get_files(target,"*.htm")

        for htm in files:
            convert_file(target,htm)

def main():
    global db
    parser = argparse.ArgumentParser(
        description=(
            'Automates fanfic downloads latest chapters using fanfic_scraper. '
            '(Currently works with fanfiction.net)'))
    parser.add_argument(
        "-c", '--config', required=False, action='store_true', help="Configures Fanfic Downloader.")

    config_load()
    db = FanficDB()

    args = parser.parse_args()

    if args.config:
        print("Calling config menu...")
    else:
        download_stories(args)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python3
import os
import sys
import shutil
import fnmatch
import subprocess
from configparser import SafeConfigParser
from datetime import date
import sqlite3
import textwrap
from pony.orm import *
from fanfic_scraper.db_pony import DataBaseLogic, Category, Fanfic
from collections import defaultdict

category=''
folder=''
dfolder=''
sfolder=''
tfile=''
editor='vi'

#Initial Values
basePath = '/home/ubuntu/OneDrive/'
arcRoot = 'FF_Archive'
db = None
db_name = 'FanfictionDB.db'
db_folder = 'Read'

class FanficDB:

    def __init__(self):
        db_path = os.path.join(basePath,arcRoot,db_folder,db_name)
        dbhandle = DataBaseLogic(db_path)

    def get_cat_id(self, categoryName):

        with db_session:
            cat = Category.get(Name=categoryName)
            ret = cat.Id

        return ret

    def set_complete(self,fic_id):

        with db_session:
            fic = Fanfic.get(Id = fic_id)
            c = fic.Complete

            if c == 1:
                c = 0
            else:
                c = 1

            fic.Complete = c

    def set_abandoned(self,fic_id):

        with db_session:
            fic = Fanfic.get(Id = fic_id)
            a = fic.Abandoned

            if a == 1:
                a = 0
            else:
                a = 1

            fic.Abandoned = a

    def get_fic_id(self,cat_id,folder):

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Category_Id == cat_id
                        and f.Folder == folder)

            for f in ff:
                ret = f.Id

        return ret

    def get_fanfic(self,cat_id,folder):
        d = defaultdict(list)
        ret = []

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Category_Id == cat_id
                        and f.Folder == folder)

            for f in ff:
                d = defaultdict(list)
                for k,v in f.to_dict().items():
                    d[k].append(v)
                ret.append(d)

        return ret[0]

def about_story():

    source = os.path.join(basePath,arcRoot,folder,category,sfolder)
    cat = category.replace('_',' ')
    cat_id = db.get_cat_id(cat)
    r = db.get_fanfic(cat_id,sfolder)

    _=os.system("clear")

    if r["Abandoned"][0] == 1:
        print("WARNING: This story is ABANDONED and will NOT be updated.")
        print("")

    print("Title: ", r["Title"][0])
    print("Author: ", r["Author"][0])
    print("Folder Name: ", r["Folder"][0])
    print("Chapter Count: ", r["Count"][0])

    if r["Complete"][0] == 1:
        print("Complete: Yes")
    else:
        print("Complete: No")

    print("Last Updated: ", r["Update_Date"][0])
    print("Summary: ", textwrap.fill(r["Description"][0]))
    print("")

    pause()

    menu_story()

def toggle_complete():

    cat = category.replace('_',' ')
    cat_id = db.get_cat_id(cat)
    fic_id = db.get_fic_id(cat_id,sfolder)

    db.set_complete(fic_id)

    menu_story()

def toggle_abandoned():

    cat = category.replace('_',' ')
    cat_id = db.get_cat_id(cat)
    fic_id = db.get_fic_id(cat_id,sfolder)

    db.set_abandoned(fic_id)

    menu_story()

def get_entry(text,value):
    #Substitute {0} in text with value.
    return text.format(value)

def pause():
    #Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")

def submenu(values,prompt):
    inc=9
    start=0
    stop=inc
    np=0
    pp=0
 
    while True:
        print(prompt)
        print('')
        for i in range(start,len(values)):
            print(i, values[i])
            j=i
            # limit number of entries to stop var.
            if i >= stop:
                # Stop offering next if the last entry is reached.
                if i < (len(values)-1):
                    np=stop+1
                    i=i+1
                    print(np, "Next Page")
                    break
        #if stop > inc, allow prev page.
        if stop > inc:
            pp=i+1
            i=i+1
            print(pp, "Prev Page")
        print('')
        #parse input into number, blank is invalid.
        try:
            sel=int(input("Enter Selection: "))
        except:
            sel = j+1
        # Validate input is valid for current menu.
        if (sel >= start) and (sel <= j):
            ret = values[sel]
            break
        else:
         # Move 1 page ahead
         if sel == np:
             start=start+10
             stop=start+9
             _=os.system("clear")
         # Move 1 page back
         elif sel == pp:
             start=start-10
             stop=start+9
             _=os.system("clear")
         else:    
             print("Unknown Option Selected!")
             pause()
             _=os.system("clear")

    return ret

def menu_editor():
    global editor

    menu = ['vi','gedit', 'geany']
    ret = submenu(menu,"Choose Editor")

    editor = ret

def menu_options():
    menu = ['Path','Editor', 'Main Menu']
    ret = submenu(menu, "Choose Option")

    if ret == "Editor":
        menu_editor()
    if ret == "Path":
        set_path()

def menu_story():

    menu = ['About Story']
    menu = menu + ['Toggle Complete', 'Toggle Abandoned']
    menu = menu + [ "Format File", "Main Menu"]
    ret = submenu(menu,"Chose Story Option")

    if ret == "About Story":
        about_story()
    if ret == "Toggle Complete":
        toggle_complete()
    if ret == "Toggle Abandoned":
        toggle_abandoned()
    if ret == "Format File":
        if tfile:
            format_file(os.path.join(basePath,arcRoot,folder,category,sfolder,tfile))
        else:
            print("File must be selected first!")

def menu_sync():

    sync = ['Download','Upload','Clean Folder','Main Menu']
    ret = submenu(sync,"Choose OneDrive Option")

    script_dir = os.path.dirname(os.path.realpath(__file__))

    if ret == "Download":
        run_script(os.path.join(script_dir,"util/sync_dl.sh"))
    if ret == "Upload":
        run_script(os.path.join(script_dir,"util/sync_ul.sh"))
    if ret == "Clean Folder":
        run_script(os.path.join(script_dir,"util/clean-onedrive"))

def choose_value(path,value):
    #Get sub-folders in path
    subdirs = next(os.walk(path))[1]
    subdirs.sort()

    if len(subdirs) > 0:
        ret = submenu(subdirs,value)
    else:
        ret = ''
        print("No folders found!")
        pause()

    return ret

def choose_file(path,value):
    global sfolder
    global tfile

    #get files in path
    lfiles = get_files(path,'*.txt')
    ret=''

    if len(lfiles) > 0:
        lfiles.append('*.txt')
        lfiles.sort()
        ret = submenu(lfiles,value)
    else:
        lfiles = get_files(path,'*.*')
        if len(lfiles) == 0:
            os.rmdir(path)

        sfolder=''
        tfile=''
        ret=tfile

    return ret

def get_files(path,value):
    matches = [] 
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files,value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches

def get_folders(path,value):
    matches = []
    for root, dirs, files in os.walk(path):
        for dirs in fnmatch.filter(files,value):
            matches.append(dirs)
    # Sort folder alphabetically
    matches.sort()

    return matches

def list_files(path,value):
    matches = get_files(path,value)
    print_list(matches)

def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches)/divisor)):
            j = divisor * (i *1)
            result = ''
            for j in range(j,j+(divisor)):
                result=result+matches[j]+'\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor)*divisor)

    result = ''
    for m in range(l,len(matches)):
        result=result+matches[m]+'\t'
    print(result)

def run_script(value):
    subprocess.call(value,shell=True)
    pause()

def format_file(value):

    print("Processing File... please wait.")

    t_file = value+".tmp"

    f1 = open(value, "r")
    f2 = open(t_file, "w")

    fdata = f1.read()
    flines = fdata.splitlines()
    for line in flines:
        data1 = textwrap.fill(line,80)
        data2 = data1.splitlines()
        for i in range(len(data2)):
            f2.write(data2[i]+"\n")
            if i == (len(data2)-1):
                f2.write("\n")
    f2.flush()
    os.fsync(f2.fileno())

    f1.close
    f2.close

    os.remove(value)
    shutil.move(t_file,value)

def read_file(value):
    subprocess.call(editor+' '+value,shell=True)

def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)

def mov_files(path,dfolder,value):
    global tfile
    # Build destination path
    dpath = os.path.join(basePath,arcRoot,dfolder,category,sfolder)
    # Create destination folder if it doesn't exist.
    ensure_dir(dpath)

    list_files(path,value)
    confirm=input("Enter Y to proceed: ")
    if confirm.lower() == 'y':
        print("Archiving files")
        try:
            matches = get_files(path,value)
            for i in range(len(matches)):
                print("Archiving: ", matches[i])
                shutil.move(os.path.join(path,matches[i]),os.path.join(dpath,matches[i]))
            print("Archive complete.")
        except:
            print("Problem Archiving File.")

    tfile = ''
    pause()

def get_config():
    #create config dir if it doesn't exist and return path to config.
    home = os.path.expanduser("~")
    cfg = os.path.join(home,".ff")
    ensure_dir(cfg)

    return os.path.join(cfg,"config.ini")

def load_config():
    global basePath
    global folder
    global dfolder
    global sfolder
    global category
    global tfile
    global editor
    global db_folder
    global db_name
    t_editor =''
    t_path = ''
    t_dbname = ''
    t_dbfolder = ''
    config = SafeConfigParser()

    cfg = get_config()

    if os.path.isfile(cfg):
        config.read(cfg)

        try:
            t_folder = config.get('archive','sourcefolder')
            t_category = config.get('archive','category')
            t_sfolder = config.get('archive','storyfolder')
            t_dfolder = config.get('archive','destfolder')
            t_tfile = config.get('archive','textfile')
            t_editor = config.get('options','editor')
            t_path = config.get('path','path')
            t_dbfolder = config.get('path','dbfolder')
            t_dbname = config.get('path','dbname')
        except:
            i=0

        if t_editor:
            editor = t_editor
            
        if t_path:
            basePath = t_path

        if t_dbfolder:
            db_folder = t_dbfolder

        if t_dbname:
            db_name = t_dbname

        spath = os.path.join(basePath,arcRoot,t_folder)
        dpath = os.path.join(basePath,arcRoot,t_dfolder)

        #Validate parameters in config file are valid.
        if os.path.exists(dpath):
            dfolder = t_dfolder
        if os.path.exists(spath):
            folder = t_folder
            spath = os.path.join(spath,t_category)
            if os.path.exists(spath):
                category = t_category
                spath = os.path.join(spath,t_sfolder)
                if os.path.exists(spath):
                    sfolder = t_sfolder
                    sfile = os.path.join(spath,t_tfile)
                    if os.path.isfile(sfile):
                        tfile = t_tfile
             
def save_config():
    cfg = get_config()

    config = SafeConfigParser()

    config.read(cfg)

    if not config.has_section('archive'):
        config.add_section('archive')
    if not config.has_section('options'):
        config.add_section('options')
    if not config.has_section('path'):
        config.add_section('path')

    config.set('archive','sourcefolder',folder)
    config.set('archive','category',category)
    config.set('archive','storyfolder',sfolder)
    config.set('archive','destfolder',dfolder)
    config.set('archive','textfile',tfile)
    config.set('options','editor',editor)
    config.set('path','path',basePath)
    config.set('path','rootfolder',arcRoot)
    config.set('path','dbfolder',db_folder)
    config.set('path','dbname',db_name)

    with open(cfg, 'w') as f:
        config.write(f)


def set_path():
    global basePath

    msg = 'Sync Folder: {0}'
    print(msg.format(basePath))

    root=input("Enter root path to sync folder: ")

    if os.path.exists(root):
        basePath = root
    else:
        print("Invalid path, please try again.")
        pause()
        _=os.system("clear")
        set_path()

def mainmenu():
    global basePath
    global folder
    global dfolder
    global sfolder
    global category
    global tfile

    load_config()

    while True:
        menu = {}
        menu[1] = get_entry("Source Folder: {0}",folder)
        menu[2] = get_entry("Source Category: {0}",category)
        menu[3] = get_entry("Story: {0}",sfolder)
        menu[4] = get_entry("Dest Folder: {0}",dfolder+"/"+category+"/"+sfolder)
        menu[5] = get_entry("Text File: {0}",tfile)
        menu[6] = "Read File"
        menu[7] = "Move File"
        menu[8] = "Story Options"
        menu[9] = "Sync Options"
        menu[10] = "Config Options"
        menu[11] = "Exit"
        options=menu.keys()
        _=os.system("clear")
        print("Manage FanFiction Archive")
        print('')
        print(get_entry("Sync Folder: {0}",basePath))
        print('')
        for entry in options:
            opt = '%02d' % entry
            print(opt, menu[entry])
        print('')
        selection=input("Please Select: ") 
        if selection =='1': 
            root = os.path.join(basePath,arcRoot)
            if not os.path.exists(root):
                print("Please set a valid Sync Folder first!")
                basePath = ''
            else:
                folder = choose_value(os.path.join(basePath,arcRoot),"Choose Folder:")
        elif selection == '2':
            if not folder:
                print("Please choose a folder first!")
                pause()
            else:
                category = choose_value(os.path.join(basePath,arcRoot,folder),"Choose Category:")
                sfolder = ''
                tfile = ''
        elif selection == '3':
            if not category:
                print("Please choose a category first!")
                pause()
            else:
                sfolder = choose_value(os.path.join(basePath,arcRoot,folder,category),"Choose Story Folder:")
                tfile = ''
        elif selection == '4':
            if not sfolder:
                print("Please choose a story first!")
                pause()
            else:
                 dfolder = choose_value(os.path.join(basePath,arcRoot),"Choose Destination Folder:")
        elif selection == '5':
            if not sfolder:
                print("Please choose a story first!")
                pause()
            else:
                 tfile = choose_file(os.path.join(basePath,arcRoot,folder,category,sfolder),"Choose File:")
        elif selection == '6':
            if not tfile:
                print("Please select a file first!")
                pause()
            else:
                read_file(os.path.join(basePath,arcRoot,folder,category,sfolder,tfile))
        elif selection == '7':
            if not tfile:
                print("Please select a file first!")
                pause()
            else:
                mov_files(os.path.join(basePath,arcRoot,folder,category,sfolder),dfolder,tfile)
        elif selection == '8':
            if not sfolder:
                print("Please choose a story first!")
                pause()
            else:
                menu_story()
        elif selection == '9':
            menu_sync()
        elif selection == '10':
            menu_options()
        elif selection == '11':
            save_config()
            break
        else: 
            print("Unknown Option Selected!")
            pause()

def main():
    global db

    #Initialize DB
    db = FanficDB()
    #Call Main Menu
    mainmenu()

if __name__ == "__main__":
    sys.exit(main())



"""Define current fanfic class based on url."""

from fanfic_scraper.extractors.fanfictionnet import FanfictionNetFanfic

def fanfic(fanfic_url, args, verify_https):
    """Send the approriate class."""
    if 'fanfiction.net' in fanfic_url:
        return FanfictionNetFanfic(fanfic_url, args, verify_https)



from setuptools import setup

setup(name='fanfic-scraper',
      version='0.7.2',
      description='Scrapes fanfics, maintains download db, organized folders for reading.',
      url='https://github.com/KebinuChiousu/fanfic-scraper',
      download_url='https://github.com/KebinuChiousu/fanfic_scraper/archive/v0.7.2.tar.gz',
      author='Kevin Meredith',
      author_email='kevin@meredithkm.info',
      license='MIT',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5',
          'Topic :: Games/Entertainment',
      ],
      keywords='fanfic scraper',
      packages=['fanfic_scraper'],
      install_requires=[
          'beautifulsoup4==4.6.0',
          'certifi==2017.7.27.1',
          'chardet==3.0.4',
          'futures==3.1.1',
          'idna==2.6', 
          'requests==2.18.4',
          'urllib3==1.22',
          'html5lib==1.0.1',
          'html2text==2018.1.9',
          'pony==0.7.3',
          'tldextract==2.2.0'
      ],
      entry_points={
          'console_scripts': [
          'fanfic-scraper=fanfic_scraper.fanfic_scraper:main',
          'ff=fanfic_scraper.ff:main',
          'ffdl=fanfic_scraper.ffdl:main',
          'ff-fif=fanfic_scraper.ff_fif:main',
          'ff-h2t=fanfic_scraper.ff_h2t:main',
          'ff-rf=fanfic_scraper.ff_rf:main'
        ],
      },
      include_package_data=True,
      zip_safe=False)



#!/usr/bin/python3

import os
import sys
import argparse
import shutil
import fnmatch
import re
from collections import defaultdict

def pause():
    #Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")

def print_format(text,value):
    #Substitute {0} in text with value.
    print(text.format(value))

def get_files(path,value):
    matches = [] 
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files,value):
            f = root+'/'+filename
            matches.append(f)

    # Sort files alphabetically
    matches.sort()

    return matches

def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)

def list_files(path,value):
    matches = get_files(path,value)
    print_list(matches)

def print_list(matches):
    divisor = 6
    result = ''
 
    for i in range(int(len(matches)/divisor)):
            j = divisor * (i *1)
            result = ''
            for j in range(j,j+(divisor)):
                result=result+matches[j]+'\t'
            print(result)
    
    k = len(matches) % divisor
    l = (int(len(matches) / divisor)*divisor)

    result = ''
    for m in range(l,len(matches)):
        result=result+matches[m]+'\t'
    print(result)

def get_prefix(file):
    matches = []
    r = re.compile("([a-zA-Z]+[0-9]+[a-zA-Z]+-)|([a-zA-Z]+[0-9]+-)|([a-zA-Z]+)")

    m = r.match(file)
    
    if m:
        if (m.group(1)):
            t = m.group(1)
        elif (m.group(2)):
            t = m.group(2)
        elif (m.group(3)):
            t = m.group(3)

    return t

def main():

    """Rename fanfic chapters."""
    parser = argparse.ArgumentParser(
        description=('Utility to rename fanfic chapters'))

    parser.add_argument('folder', help='fanfic folder to process')

    args = parser.parse_args()

    source = args.folder

    txt = get_files(source,"*.txt")
    htm = get_files(source,"*.htm")

    files = txt+htm

    for f in files:
        d = os.path.dirname(f)
        f = os.path.basename(f)
        p = get_prefix(f)
        n = int(f.replace(p,'').split('.')[0])
        p = p.split('-')[0] + '-'
        num = '%03d' % n
        ext = f.replace(p,'').split('.')[1]
        target = p+num+'.'+ext
        print("Renaming: " + f + " to " + target)
        shutil.move(os.path.join(d,f),os.path.join(d,target))

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python3

import os
import sys
import shutil
import fnmatch
import subprocess

def pause():
    #Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")

def print_format(text,value):
    #Substitute {0} in text with value.
    print(text.format(value))

def get_files(path,value):
    matches = [] 
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files,value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches

def print_list(matches):
    divisor = 6
    result = ''
 
    for i in range(int(len(matches)/divisor)):
            j = divisor * (i *1)
            result = ''
            for j in range(j,j+(divisor)):
                result=result+matches[j]+'\t'
            print(result)
    
    k = len(matches) % divisor
    l = (int(len(matches) / divisor)*divisor)

    result = ''
    for m in range(l,len(matches)):
        result=result+matches[m]+'\t'
    print(result)

def convert_file(source,htm):

    txt = htm.split('.')[0]+'.txt'
    ifile = [os.path.abspath(os.path.join(source,htm))]
    ofile = os.path.abspath(os.path.join(source,txt))
    cmd = ['html2text','--ignore-emphasis','-b 80']
    msg = 'Converting file: {0} to text file: {1}'

    print(msg.format(htm,txt))

    with open(ofile, "w") as outfile:
        subprocess.call(cmd+ifile , stdout=outfile)

    for i in ifile:
        os.remove(i)

def main(argv):
    if len(argv) < 2:
        sys.stderr.write("Usage: %s <folder_path>" % (argv[0],))
        return 1

    if not os.path.exists(argv[1]):
        sys.stderr.write("ERROR: Folder %r was not found!" % (argv[1],))
        return 1
    
    source = argv[1]
    
    files = get_files(source,"*.htm")

    #print_list(files)

    source = os.path.realpath(source)
    
    for htm in files:
        convert_file(source,htm)

if __name__ == "__main__":
    sys.exit(main(sys.argv))
